using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using TestSearch;
using System.Diagnostics;
using System.IO;

namespace GuiSearch
{
    public partial class Form1 : Form
    {
        IndexMethod _currentFilter = IndexMethod.Naive;
        IIndexer<string> _indexer = null;
        private string _filename;
        QuickListController _controller;

        public Form1()
        {
            InitializeComponent();

            _controller = new QuickListController(this.listView1);

            radioButtonNaive.Checked = true;
                        
        }

        private void CreateIndex()
        {
            if (_filename == null)
            {
                labelStatus.Text = "No file selected";
                return;
            }

            string[] lines = File.ReadAllLines(_filename);
            _controller.AllLines = lines;

            HighPerformanceTimer createTimer = new HighPerformanceTimer();
            long memoryUseBefore = GC.GetTotalMemory(true);
            //get the indexer we need to use
            
            switch (_currentFilter)
            {
                case IndexMethod.Naive:
                    _indexer = new NaiveIndexer<string>();
                    break;
                case IndexMethod.SubString:
                    _indexer = new SubStringIndexer<string>(lines.Length, 3);
                    break;
                case IndexMethod.Trie:
                    _indexer = new TrieIndexer<string>(3);
                    break;
                default:
                    Debug.Assert(false);
                    break;
            }
            createTimer.Start();
            //add everything to the index
            for (Int32 currentItem = 0; currentItem < lines.Length; currentItem++)
            {
                _indexer.AddItem(lines[currentItem], lines[currentItem], (UInt32)currentItem);
            }

            createTimer.Stop();
            long memoryUseAfter = GC.GetTotalMemory(true);
            long memoryDelta = memoryUseAfter - memoryUseBefore;
            labelStatus.Text = string.Format("Create: {0:N5}s, {1:N0} lines, {2:N0} bytes", createTimer.Duration,lines.Length,memoryDelta);
            _controller.Indexer = _indexer;
        }

        private void OnFilterMethodChecked(object sender, EventArgs e)
        {
            if (radioButtonNaive.Checked)
            {
                _currentFilter = IndexMethod.Naive;
            }
            else if (radioButtonSubString.Checked)
            {
                _currentFilter = IndexMethod.SubString;
            }
            else if (radioButtonTrie.Checked)
            {
                _currentFilter = IndexMethod.Trie;
            }
            else
            {
                Debug.Assert(false);
                _currentFilter = IndexMethod.None;
            }
            CreateIndex();
            _controller.Indexer = _indexer;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = "Text File (*.txt)|*.txt|All Files (*.*)|*.*";
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                _filename = ofd.FileName;
                CreateIndex();
                _controller.RefreshList();
            }
        }

        private void OnFilterTextChanged(object sender, EventArgs e)
        {
            HighPerformanceTimer timer = new HighPerformanceTimer();
            
            _controller.FilterText = textBoxFilter.Text;
            timer.Start();
            _controller.RefreshList();
            timer.Stop();
            labelStatus.Text = string.Format("Found {0:N0} items in {1:N5}s", _controller.CurrentResults.Count, timer.Duration);
        }
    }
}